@@ -1,13 +1,14 @@
 import time
 # Non standard libraries
 from launchpy import config, connector
+from typing import Union
 
 class Library:
     """
     A class that handle the library in a Launch environment.
     """
 
-    def __init__(self, data: dict,config_object:dict=config.config_object,header:dict=config.header):
+    def __init__(self, data: Union[str,dict],config_object:dict=config.config_object,header:dict=config.header):
         """
         The instantiator for the library.
         Arguments:
@@ -19,11 +20,12 @@ class Library:
             config_object=config_object, header=header)
         self.header = self.connector.header
         self.endpoint = config.endpoints['global']
+        if type(data) == str:
+            data = self.connector.getData("https://reactor.adobe.io/libraries/"+data).get('data')
         self.id = data['id']
         self.name = data['attributes']['name']
         self.state = data['attributes']['state']
         self.build_required = data['attributes']['build_required']
-        self.builds = data['relationships']['builds']['links']['related']
         self._DataElements = config.endpoints['global'] + \
             '/libraries/'+data['id']+'/data_elements'
         self._Extensions = config.endpoints['global'] + \
@@ -101,6 +103,23 @@ class Library:
             return dataOrigin
         self.relationships['extensions'] = data
         return data
+    
+    def getBuilds(self)->list:
+        """
+        Retrieve the last builds.
+        Return a list of build
+        """
+        params = {'page[number]':1}
+        res = self.connector.getData(self._Builds,params=params)
+        builds = res.get('data',[])
+        next_page = res.get('meta',{}).get('pagination',{}).get('next_page',None)
+        while next_page is not None:
+            params['page[number]'] +=1
+            res = self.connector.getData(self._Builds,params=params)
+            builds += res.get('data',[])
+            next_page = res.get('meta',{}).get('pagination',{}).get('next_page',None)
+        return builds
+
 
     def getRules(self,page:int=0,pageSize:int=50,origin:bool=True)->list:
         """
@@ -177,6 +196,24 @@ class Library:
         res = self.connector.patchData(self.endpoint +url, data=obj)
         return res
 
+    def removeDataElements(self,data_element_ids:list)->dict:
+        """
+        Take a list of data elements and remove them from the library.
+        Arguments:
+            data_element_ids: REQUIRED : list of data elements id
+        """
+        if self.state != 'development':
+            print('State is not development, cannot update relationships')
+            return None
+        obj = {'data': []}
+        if type(data_element_ids) == str:
+            data_element_ids = data_element_ids.split(' ')
+        for ids in data_element_ids:
+            obj['data'].append({"id": ids,"type": "data_elements"})
+        url =  f'/libraries/{self.id}/relationships/data_elements'
+        print(obj)
+        res = self.connector.deleteData(self.endpoint +url, data=obj)
+        return res
 
     def addRules(self, rules_ids: list)->dict:
         """
@@ -217,6 +254,25 @@ class Library:
         res = self.connector.patchData(self.endpoint + url, data=obj)
         return res
 
+    def removeRules(self,rules_ids:list)->dict:
+        """
+        Remove the rules that are passed. 
+        Arguments:
+            rules_ids: REQUIRED : list of rules id
+        """
+        if self.state != 'development':
+            print('State is not development, cannot add relationships')
+            return None
+        obj = {'data': []}
+        if type(rules_ids) == str:
+            rules_ids = rules_ids.split(' ')
+        for ids in rules_ids:
+            obj['data'].append({"id": ids, "type": "rules"})
+        url = f'/libraries/{self.id}/relationships/rules'
+        res = self.connector.deleteData(self.endpoint + url, data=obj)
+        return res
+
+
     def addExtensions(self, extensions_ids: list)->object:
         """
         Take a list of extension id and attach them to the library. 
@@ -260,7 +316,7 @@ class Library:
         Save the different environments ids available. 
         It is required to use the library class.
         Arguments : 
-            environments_list : REQUIRED : list of environment retrieved by the getEnvironment method
+            environments_list : REQUIRED : list of environment retrieved by the getEnvironments method
             dev_name : OPTIONAL : Name of your dev environment. If not defined, will take the first dev environment.
         """
         for env in environments_list:
@@ -292,9 +348,11 @@ class Library:
         new_env = self.connector.getData(self.endpoint+path) 
         return new_env
     
-    def updateLibrary(self)->dict:
+    def updateLibrary(self,empty:bool=False)->dict:
         """
         Update the library
+        Arguments:
+            empty : OPTIONAL : You want to empty the library.
         """
         path = f'/libraries/{self.id}'
         data = {
@@ -306,6 +364,21 @@ class Library:
                     }
                 }
                 }
+        if True:
+            data['relationships'] = {
+                "data_elements": {
+                    "data": []
+                },
+                "rules": {
+                    "data": []
+                },
+                "extensions": {
+                    "data": []
+                },
+                "environment":{
+                    "data":{},
+                }
+            }
         res = self.connector.patchData(self.endpoint+path,data=data)
         return res
 
@@ -376,9 +449,9 @@ class Library:
         If no action is provided, it would automatically go to the next state. 
         Arguments : 
             action : OPTIONAL : action to do on the library. Possible values: 
-                - 'submit' : if state == development
-                - 'approve' : if state == submitted
-                - 'reject' : if state == submitted
+                - 'submit' : if current state == development
+                - 'approve' : if current state == submitted
+                - 'reject' : if current state == submitted
         """
         path = f'/libraries/{self.id}'
         if action == None:
@@ -397,5 +470,5 @@ class Library:
         transition = self.connector.patchData(self.endpoint+path, data=obj)
         data = transition
         self.state = data['data']['attributes']['state']
-        self.build_required = data['data']['attributes']['build_required']
+        self.build_required = True
         return data
\ No newline at end of file
