@@ -3,7 +3,6 @@ from collections import defaultdict
 from concurrent import futures
 from copy import deepcopy
 # Non standard libraries
-import pandas as pd
 from launchpy import config, connector
 from typing import IO, Union
 from .library import Library
@@ -49,7 +48,7 @@ class Property:
         self._Rules = data['links']['rules']
         self._RuleComponents = 'https://reactor.adobe.io/properties/' + \
             self.id + '/rule_components'
-        self._Host = 'https://reactor.adobe.io//properties/' + \
+        self._Host = 'https://reactor.adobe.io/properties/' + \
             data['id']+'/hosts'
         self._Note = 'https://reactor.adobe.io/notes/'
         self._Environments = data['links']['environments']
@@ -64,7 +63,7 @@ class Property:
         return str(json.dumps(self.definition, indent=4))
     
 
-    def _getExtensionPackage(self, ext_name: str, platform: str = "web", verbose: bool = False)->dict:
+    def _getExtensionPackageIdByName(self, ext_name: str, platform: str = "web", verbose: bool = False)->dict:
         """
         Retrieve extension id of the catalog from an extension name.
         It will be used later on to check for available updates. 
@@ -83,6 +82,23 @@ class Property:
             print('extension name : ' + str(data['attributes']['name']))
             print('extension id : ' + str(data['id']))
         return extension_id
+    
+    def getExtensionPackage(self, extension_id: str = None, verbose: bool = False)->dict:
+        """
+        Retrieve the extension package information based on the extension id.
+        Arguments:
+            extension_id : REQUIRED : ID of the extension package to retrieve
+            verbose : OPTIONAL : if set to True, will print the extension name and id.
+        """
+        if extension_id is None:
+            raise ValueError("You must provide an extension id")
+        path = f'/extension_packages/{extension_id}'
+        res_ext = self.connector.getData(self.endpoint+path)
+        data = res_ext['data']
+        if verbose:
+            print('extension name : ' + str(data['attributes']['name']))
+            print('extension id : ' + str(data['id']))
+        return data
 
     def getRessource(self,res_url: str = None, params: dict = None):
         """
@@ -100,8 +116,15 @@ class Property:
         """
         Retrieve the environment sets for this property
         """
+        params = {"page[number]":1}
         env = self.connector.getData(self._Environments)
         data = env['data']  # skip meta for now
+        next_page = env.get('meta',{}).get('pagination',{}).get('next_page',None)
+        while next_page is not None:
+            params['page[number]'] = next_page
+            env = self.connector.getData(self._Environments,params=params)
+            data += env.get('data',[])
+            next_page = env.get('meta',{}).get('pagination',{}).get('next_page',None)
         return data
 
     def getHost(self)->object:
@@ -139,14 +162,25 @@ class Property:
         except:
             data = extensions
         return data
+    
+    def getExtension(self,extensionId:str=None)-> object:
+        """
+        Retrieve the definiton of the extension based on its ID.
+        Arguments:
+            extensionId : REQUIRED : ID of the extension to retrieve
+        """
+        path = f"/extensions/{extensionId}"
+        extensions = self.connector.getData(self.endpoint+path)
+        return extensions['data']
 
-    def checkExtensionUpdate(self, platform: str = "web", verbose: bool = False):
+    def checkExtensionUpdate(self, name:str=None, platform: str = "web", verbose: bool = False):
         """
         Returns a dictionary of extensions with their names, ids and if there is an update. 
         If there is an update available, the id returned is the latest id (to be used for installation). 
         It can be re-use for installation and for checking for update. 
         Arguments:
-            platform : REQUIRED : if you need to look for extension on a specific platform (default web).
+            name : OPTIONAL : If you want to check for a specific extension
+            platform : OPTIONAL : if you need to look for extension on a specific platform (default web).
             verbose: OPTIONAL : if set to True, will print the different name and id of the extensions checked.
         Dictionary example: 
         {'adobe-mcid':
@@ -156,13 +190,18 @@ class Property:
         }
         """
         extensions = self.getExtensions()
-        dict_extensions = {ext['attributes']['name']: {'package_id': ext['relationships']['extension_package']['data']['id'],
+        if name is None:
+            dict_extensions = {ext['attributes']['name']: {'package_id': ext['relationships']['extension_package']['data']['id'],
                                                        'update': False,
                                                        'internal_id': ext['id']
                                                        } for ext in extensions}
+        else:
+            dict_extensions = {ext['attributes']['name']: {'package_id': ext['relationships']['extension_package']['data']['id'],
+                                                       'update': False,
+                                                       'internal_id': ext['id']
+                                                       } for ext in extensions if ext['attributes']['name'] == name}
         for name in dict_extensions:
-            new_id = self._getExtensionPackage(
-                name, platform=platform, verbose=verbose)
+            new_id = self._getExtensionPackageIdByName(name, platform=platform, verbose=verbose)
             if new_id != dict_extensions[name]['package_id']:
                 dict_extensions[name]['package_id'] = new_id
                 dict_extensions[name]['update'] = True
@@ -176,28 +215,63 @@ class Property:
             extension_id : REQUIRED : Your internal ID for this extension in your property (EX....)
             package_id : REQUIRED : new extension id for the extension (EP...)
         """
-        data = {'data': {
-            'id': extension_id,
-                "type": "extensions",
-                "relationships": {"extension_package": {
-                    "links": {
-                        "related": "https://reactor.adobe.io/extensions/"+extension_id+"/extension_package"
-                    },
-                    "data": {
-                        "id": package_id,
-                        "type": "extension_packages"
+        if extension_id is None:
+            raise ValueError("You must provide an extension id")
+        if package_id is None:
+            raise ValueError("You must provide a package id")
+        myExtension = self.getExtension(extension_id)
+        latest_package_id = myExtension.get('meta').get('upgrade_extension_package_id',None)
+        current_package_id = myExtension['relationships']['extension_package']['data']['id']
+        if latest_package_id is None:
+            settings = myExtension.get('attributes',{}).get('settings','{}')
+            attributes = {'delegate_descriptor_id': f'{myExtension.get("attributes",{}).get("name")}::extensionConfiguration::config','settings':settings}
+            data = {'data': {
+                    'id': extension_id,
+                        "type": "extensions",
+                        "relationships": {"extension_package": {
+                            "links": {
+                                "related": "https://reactor.adobe.io/extensions/"+extension_id+"/extension_package"
+                            },
+                            "data": {
+                                "id": package_id,
+                                "type": "extension_packages"
+                            }
+                        }
+                        },
+                        'meta': {'upgrade_extension_package_id': package_id}
+                        }
                     }
-                }
-                },
-                'meta': {
-                    'upgrade_extension_package_id': package_id
-                }
-                }
-                }
+        else:
+            if latest_package_id != current_package_id: ##using a different package that the one provided
+                settings = myExtension.get('attributes',{}).get('settings','{}')
+                attributes = {'delegate_descriptor_id': f'{myExtension.get("attributes",{}).get("name")}::extensionConfiguration::config','settings':settings}
+                data = {'data': {
+                    "attributes": attributes,
+                    'id': extension_id,
+                        "type": "extensions",
+                        "relationships": {"extension_package": {
+                            "links": {
+                                "related": "https://reactor.adobe.io/extensions/"+extension_id+"/extension_package"
+                            },
+                            "data": {
+                                "id": latest_package_id,
+                                "type": "extension_packages"
+                            }
+                            }
+                        }
+                        }
+                    }
+            else:
+                raise ValueError("Extension already up to date")
+
         path = f"/extensions/{extension_id}"
         res = self.connector.patchData(
             self.endpoint+path, data=data)
-        return res
+        try:
+            data = res['data']  # skip meta for now
+        except:
+            data = res
+        return data
     
     def getProfile(self)->dict:
         """
@@ -259,9 +333,9 @@ class Property:
         """
         filters = {}
         if name != None:
-            filters['filter[name]=CONTAINS '] = f"EQ {name}"
+            filters['filter[name]'] = f"EQ {name}"
         if name_contains != None:
-            filters['filter[name]=CONTAINS '] = f"CONTAINS {name_contains}"
+            filters['filter[name]'] = f"CONTAINS {name_contains}"
         if dirty != None:
             filters['filter[dirty]'] = f"EQ {str(dirty).lower()}"
         if enabled != None:
@@ -298,6 +372,27 @@ class Property:
                 'url': rule['links']['rule_components']
             }
         return data
+    
+    def extractRuleComponents(self,rule:dict=None)->list:
+        """
+        Extract the rule component based on the rule definition.
+        Arguments:
+            rule : REQUIRED : The rule definition
+        """
+        if rule is None:
+            raise ValueError("No rule definition provided")
+        params={'page[number]':0}
+        rc_endpoint = rule['relationships']['rule_components']['links']['related']
+        page1 = self.getRessource(rc_endpoint)
+        data = page1.get('data',{})
+        next_page = page1.get('meta',{}).get('pagination',{}).get('next_page',None)
+        while next_page is not None:
+            params['page[number]'] = next_page
+            pageN = self.connector.getData(rc_endpoint,params=params)
+            data += pageN.get('data',[])
+            next_page = pageN.get('meta',{}).get('pagination',{}).get('next_page',None)
+        return data
+
 
     def getRuleComponents(self, verbose:bool=False,**kwargs)->dict:
         """
@@ -413,6 +508,8 @@ class Property:
             raise ValueError('Require a Data Element ID')
         path = f"/data_elements/{dataElementId}"
         res = self.connector.getData(self.endpoint+path)
+        if 'data' in res.keys():
+            return res['data']
         return res
 
     def searchDataElements(self, name: str = None, enabled: bool = None, published: bool = None, dirty: bool = None, **kwargs)->object:
@@ -506,6 +603,20 @@ class Property:
             data = data + append_data
         return data
 
+    def getLibrary(self,libraryId:str=None)->dict:
+        """
+        get a library based on its ID.
+        Arguments:
+            libraryId : REQUIRED : Library ID to be retrieved
+        """
+        if libraryId is None:
+            raise ValueError("Require a library ID")
+        path = f"/libraries/{libraryId}"
+        res = self.connector.getData(self.endpoint+path)
+        if 'data' in res.keys():
+            return res['data']
+        return res
+
     def getNotes(self, data: object)->list:
         """
         Retrieve the note associated with the object pass to the method. Returns list.
@@ -539,7 +650,7 @@ class Property:
             data = data + append_data
         return data
 
-    def createExtensions(self, extension_id: str, settings: str = None, descriptor: str = None, **kwargs)-> object:
+    def createExtension(self, extension_id: str, settings: str = None, descriptor: str = None, **kwargs)-> object:
         """
         Create an extension in your property. Your extension_id argument should be the latest one extension id available.
         Arguments : 
@@ -801,7 +912,7 @@ class Property:
         """
         update the extension with the information provided in the argument.
         argument: 
-            attr_dict : REQUIRED : dictionary that will be passed to Launch for update
+            attr_dict : REQUIRED : attribute dictionary/object that will be passed to Launch for update
         """
         obj = {
             "data": {
@@ -877,27 +988,34 @@ class Property:
                 lastPage=True
         return data
 
-    def getLatestPublishedVersion(self,revisions:list=None)->dict:
+    def getLatestPublishedVersion(self,revisions:Union[list,dict]=None)->dict:
         """
         Find the latest published version of a component based on the list of revisions retrieved via getRevisions methods.
         Arguments:
-            revisions : REQUIRED : list of revisions
+            revisions : REQUIRED : list of revisions or dictionary of the component.
         """
         if revisions is None:
-            raise ValueError('Require a list of revisions')
+            raise ValueError('Require a list of revisions at least')
+        if type(revisions) == dict:
+            revisions = self.getRevisions(element=revisions)
         publishedIndexVersions:dict = {index:rev['attributes']['revision_number'] for index, rev in enumerate(revisions) if rev['attributes']['published'] == True}
+        maxVersion = max(e['attributes']['revision_number'] for e in revisions)
         if len(publishedIndexVersions) == 0:
-            raise IndexError("You want to retrieve a published version of the component.\nBut no published version can be found. Please check if your component has been published")
+            raise IndexError(f"You want to retrieve a published version of the component.\nBut no published version can be found in {self.name}. Please check if your component has been published")
         maxRevisionIndex = [index for index,value in publishedIndexVersions.items() if value == max(list(publishedIndexVersions.values()))][0]
-        return revisions[maxRevisionIndex]
-
+        publishedVersion = revisions[maxRevisionIndex]
+        if maxVersion == publishedVersion['attributes']['revision_number']:
+            publishedVersion['attributes']['latest'] = True
+        else:
+            publishedVersion['attributes']['latest'] = False
+        return publishedVersion
 
     def reviseDataElement(self, dataElement_id: str)->dict:
         """
         Update the data element information based on the information provided.
         arguments: 
             dataElement_id : REQUIRED : Data Element ID
-            attr_dict : REQUIRED : dictionary that will be passed to Launch for update
+            attr_dict : REQUIRED : attributes dictionary/object that will be passed to Launch for update
         """
         obj = {
             "data": {
@@ -914,7 +1032,7 @@ class Property:
         data = dataElements
         return data
 
-    def getRule(self, rule_id: str=None)->object:
+    def getRule(self, rule_id: str=None)->dict:
         """
         Update the rule based on elements passed in attr_dict. 
         arguments: 
@@ -927,6 +1045,8 @@ class Property:
         path = f'/rules/{rule_id}'
         rule = self.connector.getData(
             self.endpoint+path)
+        if 'data' in rule.keys():
+            return rule['data']
         return rule
 
     def updateRule(self, rule_id: str, attr_dict: dict)->dict:
@@ -979,13 +1099,13 @@ class Property:
     
     def updateCustomCode(self,rc_id:str=None,customCode:Union[str,IO]=None,encoding:str='utf-8')->dict:
         """
-        Update the custom code of a rule (analytics or core)
+        Update the custom code of a component (analytics action or core action or data element).
         Arguments:
-            rc_id : REQUIRED : Rule Component ID
-            customCode : REQUIRED : code to be updated in the ruleComponent.2 options:
-                javaScript file; example : "myCode.js" -> ".js" suffix is important
+            comp_id : REQUIRED : Component ID
+            customCode : REQUIRED : code to be updated in the component.2 options:
+                javaScript file; example : "myCode.js" -> ".js" suffix is required.
                 string; the code you want to write as a string.
-            encoding: OPTIONAL : encoding to read the JS file. Default (utf-16)
+            encoding: OPTIONAL : encoding to read the JS file. Default (utf-8)
         """
         if rc_id is None:
             raise ValueError('Require a ruleComponent ID')
@@ -1008,7 +1128,7 @@ class Property:
         return res
 
 
-    def updateDataElement(self, dataElement_id: str, attr_dict: object, **kwargs)->object:
+    def updateDataElement(self, dataElement_id: str, attr_dict: object, **kwargs)->dict:
         """
         Update the data element information based on the information provided.
         arguments: 
@@ -1031,7 +1151,21 @@ class Property:
             data = dataElements
         return data
 
-    def updateEnvironment(self, name: str, env_id: str, **kwargs)->object:
+    def updateDataElementCode(self,dataElementId:str=None,code:str=None)->dict:
+        """
+        Update a data element custom code by passing the data element ID and the code as strng you want to upload.
+        Arguments:
+            dataElementId : REQUIRED : The data element ID
+            code : REQUIED : The code stringify
+        """
+        de = self.getDataElement(dataElementId)
+        attr = deepcopy(de['attributes'])
+        attr['settings'] = json.dumps({'source' : code})
+        print(json.dumps(attr,indent=2))
+        newDE = self.updateDataElement(dataElementId,attr)
+        return newDE
+
+    def updateEnvironment(self, name: str, env_id: str, **kwargs)->dict:
         """
         Update an environment. Note :only support name change.
         Arguments:
@@ -1056,12 +1190,14 @@ class Property:
             data = env
         return data
 
-    def updateExtensions(self, extension_id, attr_dict: dict, **kwargs)-> object:
+    def updateExtension(self, extension_id, attr_dict: dict, **kwargs)-> object:
         """
         update the extension with the information provided in the argument.
         argument: 
             extension_id : REQUIRED : the extension id
             attr_dict : REQUIRED : dictionary that will be passed to Launch for update
+        possible kwargs:
+            upgradeLatest : OPTIONAL : Boolean if you want to update the relationships of the extension to the new version
         """
         obj = {
             "data": {
@@ -1070,8 +1206,7 @@ class Property:
                 "type": "extensions"
             }
         }
-        extensions = self.connector.patchData(
-            config.endpoints['global']+'/extensions/'+extension_id, data=obj)
+        extensions = self.connector.patchData(self.endpoint+'/extensions/'+extension_id, data=obj)
         try:
             data = extensions['data']
         except:
@@ -1118,7 +1253,7 @@ class Property:
             'https://reactor.adobe.io/rule_components/'+rc_id)
         return data
 
-    def deleteEnvironments(self, env_id: str)->str:
+    def deleteEnvironment(self, env_id: str)->str:
         """
         Delete the environment based on the id.  
         Arguments: 
@@ -1128,11 +1263,56 @@ class Property:
             'https://reactor.adobe.io/environments/'+env_id)
         return data
 
+    def deleteLibrary(self,library:str=None,components:bool=False)->str:
+        """
+        Delete a Library based on its name or ID.
+        Arguments:
+            library : REQUIRED : Either the ID of the library or the name of the library.
+            components : OPTIONAL : If set to True, it will try to delete all components inside that library.
+        """
+        if library is None:
+            raise ValueError("Require at least library ID")
+        libraries = self.getLibraries()
+        librariesIds = [lib['id'] for lib in libraries]
+        librariesNameId = {lib['attributes']['name']:lib['id'] for lib in libraries}
+        if library in librariesIds:
+            libraryId = library
+        else:
+            libraryId = librariesNameId[library]
+        path = f"/libraries/{libraryId}"
+        if components==True:  
+            myLib = self.getLibrary(libraryId)
+            libClass = Library(myLib)
+            rules = libClass.getRules()
+            dataelements = libClass.getDataElements()
+        res = self.connector.deleteData('https://reactor.adobe.io/'+path)
+        for rule in rules:
+            self.deleteRule(rule['id'])
+        for de in dataelements:
+            self.deleteDataElement(de['id'])
+        return res
+    
+    def getProductionEndpoint(self)->dict:
+        """
+        Returns the production library URL to use on the website
+        """
+        envs = self.getEnvironments()
+        prod = [env for env in envs if env['attributes']['stage'] == 'production'][0]
+        return prod['meta']['script_sources'][0]["license_path"]
+
+    def getStagingEndpoint(self)->dict:
+        """
+        Returns the staging library URL to use on the website
+        """
+        envs = self.getEnvironments()
+        staging = [env for env in envs if env['attributes']['stage'] == 'staging'][0]
+        return staging['meta']['script_sources'][0]["license_path"]
+
 def extensionsInfo(data: list)->dict:
     """
     Return a dictionary from the list provided from the extensions request.
     Arguments: 
-        - data : REQUIRED : list information returned by the getExtension method. 
+        data : REQUIRED : list information returned by the getExtension method. 
     """
     extensions = {}
     for extension in data:
@@ -1159,7 +1339,7 @@ def rulesInfo(data: list)-> dict:
     """
     Return a dictionary from the list provided from the rules request.
     Arguments : 
-        - data : REQUIRED : list information returned by the getRules method. 
+        data : REQUIRED : list information returned by the getRules method. 
     """
     rules = defaultdict(None)
     for rule in data:
@@ -1182,7 +1362,7 @@ def ruleComponentInfo(data: list)->dict:
     """
     Return a dictionary from the list provided from the rules component request.
     Arguments : 
-        - data : REQUIRED : list information returned by the getRuleComponent method. 
+        data : REQUIRED : list information returned by the getRuleComponent method. 
     """
     components = {}
     for component in data:
@@ -1206,7 +1386,7 @@ def dataElementInfo(data: list)->dict:
     """
     return information about data elements as dictionary.
     arguments : 
-        - data : list return by the getDataElement value
+        data : list return by the getDataElement value
     """
     elements = defaultdict(None)
     for element in data:
@@ -1474,118 +1654,6 @@ def copySettings(data: object)->object:
     return obj
 
 
-class Translator:
-    """
-    A class to store the translator dataframe for extensions ids. 
-    It has multiple methods, you should set the Extensions and the Rules. 
-    1. setBaseExtensions
-    2. extendExtensions
-    --> You can use the translate method to translate Data Element settings 
-    3. setBaseRules
-    4. extendRules
-    --> You can use the translate method to translate Rule Components settings 
-    """
-
-    def __init__(self):
-        self.rules = pd.DataFrame()
-        self.extensions = pd.DataFrame()
-
-    def setBaseExtensions(self, base_property_extensions: object, property_name: str):
-        """
-        Pass all the extensions from the base property to start building the table. 
-        Arguments: 
-            base_property : REQUIRED : list of all extensions retrieve through getExtensions method
-            property_name : REQUIRED : name of your base property.
-        """
-        df = pd.DataFrame(extensionsInfo(base_property_extensions)).T
-        df = pd.DataFrame(df['id'])
-        df.columns = [property_name]
-        self.extensions = df
-
-    def extendExtensions(self, new_property_extensions: object, new_prop_name: str)-> None:
-        """
-        Add the extensions id from a target property.
-        Arguments: 
-            new_property_extensions: REQUIRED : the extension list from your target property. 
-            new_prop_name : REQUIRED : target property name. 
-        """
-        df = pd.DataFrame(extensionsInfo(new_property_extensions)).T
-        df = pd.DataFrame(df['id'])
-        self.extensions[new_prop_name] = df
-        return self.extensions
-
-    def setBaseRules(self, base_property_rules: object, property_name: str):
-        """
-        Pass all the rules from the base property to start building the table. 
-        Arguments: 
-            base_property : REQUIRED : list of all rules retrieve through getExtensions method
-            property_name : REQUIRED : name of your base property.
-        """
-        df = pd.DataFrame(rulesInfo(base_property_rules)).T
-        df = pd.DataFrame(df['id'])
-        df.columns = [property_name]
-        self.rules = df
-
-    def extendRules(self, new_property_rules: object, new_prop_name: str):
-        """
-        Add the extensions id from a target property.
-        Arguments: 
-            new_property_rules: REQUIRED : the rules list from your target property. 
-            new_prop_name : REQUIRED : target property name. 
-        """
-        df = pd.DataFrame(rulesInfo(new_property_rules)).T
-        df = pd.DataFrame(df['id'])
-        self.rules[new_prop_name] = df
-        return self.rules
-
-    def translate(self, target_property: str, data_element: dict = None, rule_component: dict = None)->dict:
-        """
-        change the id from the base element to the new property. 
-        Pre checked should be done beforehands (updating Extension & Rules elements)
-        Arguments: 
-            target_property : REQUIRED : property that is targeted to translate the element to
-            data_element : OPTIONAL : if the elements passed are data elements
-            rule_component : OPTIONAL : if the elements passed are rule components
-        """
-        if self.extensions.empty == True:
-            raise AttributeError(
-                "You didn't import the base extensions or the target extensions")
-        if data_element is not None:
-            new_de = deepcopy(data_element)
-            base_id = new_de['extension']['id']
-            row = self.extensions[self.extensions.iloc[:, 0]
-                                  == base_id].index.values[0]
-            new_value = self.extensions.loc[row, target_property]
-            new_de['extension']['id'] = new_value
-            return new_de
-        elif rule_component is not None:
-            if self.rules.empty == True:
-                print(
-                    "The rules have not been imported, the rule id needs to be changed")
-            new_rc = deepcopy(rule_component)
-            base_id = new_rc['extension']['id']
-            row = self.extensions[self.extensions.eq(
-                base_id).any(1)].index.values[0]
-            new_value = self.extensions.loc[row, target_property]
-            # print(f"name : {rule_component['rule_name']}")
-            # print(f"old_id : {base_id}")
-            # print(f"new_id : {new_value}")
-            new_rc['extension']['id'] = new_value
-            if self.rules.empty == False:
-                new_rc['rule_setting'] = {
-                    'data': [{
-                        'id': self.rules.loc[rule_component['rule_name'], target_property],
-                        'type':'rules'}
-                    ]}
-                new_rc['rule_id'] = self.rules.loc[rule_component['rule_name'],
-                                                   target_property]
-            else:
-                print(
-                    "You didn't load the rules. Please use setExtensions and setRules, and extendExtensions and extendRules")
-                del new_rc['rules']
-            return new_rc
-
-
 def extractAnalyticsCode(rcSettings: str, save: bool = False, filename: str = None,encoding:str='utf-8')->None:
     """
     Extract the custom code of the rule and save it in a file.
@@ -1603,4 +1671,4 @@ def extractAnalyticsCode(rcSettings: str, save: bool = False, filename: str = No
         filename = filename.replace('/', '_').replace('|', '_')
         if save:
             saveFile(json_code,filename,type='js',encoding=encoding)
-        return json_code
\ No newline at end of file
+        return json_code
